from django.apps import AppConfig


class LoginRequiredConfig(AppConfig):
    name = "login_required"

    def ready(self):
        from django.contrib.auth import views

        login_not_required_views = (
            views.LoginView,
            views.PasswordResetView,
            views.PasswordResetDoneView,
            views.PasswordResetConfirmView,
            views.PasswordResetCompleteView,
        )
        for view in login_not_required_views:
            view.login_required = False



import os

from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), "README.md")) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name="django-login-required-middleware",
    version="0.9.0",
    packages=[
        "login_required",
    ],
    include_package_data=True,
    license="MIT",
    description="Requires login to all requests through middleware.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/CleitonDeLima/django-login-required-middleware",
    author="Cleiton de Lima",
    author_email="cleiton.limapin@gmail.com",
    classifiers=[
        "Environment :: Web Environment",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Framework :: Django",
        "Framework :: Django :: 3.2",
        "Framework :: Django :: 4.0",
        "Framework :: Django :: 4.1",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
)



import re

from django.conf import settings
from django.contrib.auth.middleware import AuthenticationMiddleware
from django.contrib.auth.views import redirect_to_login
from django.http import Http404
from django.urls import resolve
from django.contrib.auth import REDIRECT_FIELD_NAME as REDIRECT_FIELD_NAME_DEFAULT

IGNORE_PATHS = [
    re.compile(url) for url in getattr(settings, "LOGIN_REQUIRED_IGNORE_PATHS", [])
]

IGNORE_VIEW_NAMES = list(getattr(settings, "LOGIN_REQUIRED_IGNORE_VIEW_NAMES", []))

REDIRECT_FIELD_NAME = getattr(settings, "LOGIN_REQUIRED_REDIRECT_FIELD_NAME", REDIRECT_FIELD_NAME_DEFAULT)


class LoginRequiredMiddleware(AuthenticationMiddleware):
    @staticmethod
    def _login_required(request):
        if request.user.is_authenticated:
            return None

        path = request.path
        if any(url.match(path) for url in IGNORE_PATHS):
            return None

        try:
            resolver = resolve(path)
        except Http404:
            return redirect_to_login(request.get_full_path())

        view_func = resolver.func

        if not getattr(view_func, "login_required", True):
            return None

        view_class = getattr(view_func, "view_class", None)
        if view_class and not getattr(view_class, "login_required", True):
            return None

        if resolver.view_name in IGNORE_VIEW_NAMES:
            return None

        return redirect_to_login(request.get_full_path(), redirect_field_name=REDIRECT_FIELD_NAME)

    def process_request(self, request):
        """
        Use process_request instead of defining __call__ directly;
        Django's middleware layer will process_request in a coroutine in __acall__ if it detects an async context.
        Otherwise, it will use __call__.
        https://github.com/django/django/blob/acde91745656a852a15db7611c08cabf93bb735b/django/utils/deprecation.py#L88-L148
        """
        return self._login_required(request)



default_app_config = "login_required.apps.LoginRequiredConfig"


def login_not_required(view_func):
    """
    Decorator for views that marks that the view is accessible by
    unauthenticated users.
    """
    view_func.login_required = False
    return view_func


class LoginNotRequiredMixin:
    """
    Mixin for CBV that marks that the view is accessible by
    unauthenticated users.
    """

    login_required = False
